﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "Ammo/LMAmmo.h"
#include "LMWeaponState.h"
#include "GameFramework/Actor.h"
#include "LMWeaponBase.generated.h"


class ALMPlayer;

UCLASS()
class LEGUMEMIX_API ALMWeaponBase : public AActor
{
	GENERATED_BODY()

public:
	ALMWeaponBase();
	virtual void BeginPlay() override;
	void Setup(USkeletalMeshComponent* Mesh, AActor* CharOwner);

public:
	UFUNCTION(BlueprintCallable)
	virtual void Reload();
	
	UFUNCTION(BlueprintCallable)
	virtual void PrimaryFire();

	UFUNCTION(BlueprintCallable)
	void PlaySound(USoundWave* Sound, bool Replacing = false);

	UFUNCTION(BlueprintCallable)
	void PlayAnimation(UAnimMontage* Animation);

	/** The socket to attach the weapon to.
	 * @return The socket.
	 */
	UFUNCTION(BlueprintCallable)
	FName GetAttachmentSocketName() const { return WeaponSocket; }
	
	UFUNCTION(BlueprintCallable, BlueprintImplementableEvent)
	void OnDefaultReload();

private:
	void DefaultReload();

protected: /* Weapon Data */
	/** The sound to play when firing. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite,Category="Legumix|Sounds")
	TObjectPtr<USoundWave> FireSound;

	UPROPERTY(EditAnywhere, BlueprintReadWrite,Category="Legumix|Sounds")
	TObjectPtr<USoundWave> ReloadSound;

	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Animations", meta=(AllowPrivateAccess=true))
	TObjectPtr<UAnimMontage> PrimaryFireAnimation;

	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Legumix|Animations", meta=(AllowPrivateAccess=true))
	TObjectPtr<UAnimMontage> ReloadAnimation;

	/** The flat damage number of one bullet. */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	float Damage = 10.f;

	/** The max angle deviation of a bullet from the original ray. In degrees.*/
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true, UIMin=0, UIMax=180))
	float WeaponSpread = 10;

	/** The max distance (cm) between the origin and the target before doing minimal damage. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	float MaxDistance = 100000.f;

	/** The damage falloff distance in a 0-1 scale. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	FFloatCurve DamageFalloff;
	
	/** The type of ammo used by this weapon. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	EAmmoType AmmoType;

	/** The current number of ammo in the clip. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	int ClipAmmo;
	
	/** The maximum amount of ammo in a clip. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category="Legumix|Weapon", meta=(AllowPrivateAccess=true))
	int MaxClip;
	
	UPROPERTY(VisibleInstanceOnly, BlueprintReadWrite, Category=Legumix)
	TObjectPtr<ALMPlayer> Player;

	/** The current state of the weapon. */
	UPROPERTY(VisibleInstanceOnly, BlueprintReadWrite, Category="Legumix|Weapon")
	EWeaponState State;
	
private:
	/** An optional socket to attach the weapon to. */
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category=Legumix, meta=(AllowPrivateAccess=true))
	FName WeaponSocket;

private: /* Components */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<UAudioComponent> AudioComponent;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category=Legumix, meta=(AllowPrivateAccess=true))
	TObjectPtr<USkeletalMeshComponent> WeaponMesh;
};

